@@ -14,6 +14,7 @@ Notes:
 
 
 """
+import contextlib
 import copy
 import os.path
 import subprocess
@@ -48,8 +49,9 @@ class TestCanBus(unittest.TestCase):
 
     # Scaffolding #
 
-    NUMBER_OF_LOOPS = 10000
-    FRAME_SENDER_SPACING_MILLISECONDS = 0.14  # Approx 90 % busload at 500 kbit/s
+    NUMBER_OF_LOOPS = 1000
+    #FRAME_SENDER_SPACING_MILLISECONDS = 0.14  # Approx 90 % busload at 500 kbit/s  Beaglebone is too slow for this
+    FRAME_SENDER_SPACING_MILLISECONDS = 1
     FRAME_NUMBER_OF_DATABYTES = 8
 
     OUTPUT_FILENAME_4 = "test_out_4_TEMPORARY.kcd"
@@ -76,41 +78,67 @@ class TestCanBus(unittest.TestCase):
         except (AttributeError, ProcessLookupError) as _:
             pass
 
+        for filename in [self.OUTPUT_FILENAME_4,
+                         self.OUTPUT_FILENAME_5,
+                         self.OUTPUT_FILENAME_6,
+                         self.OUTPUT_FILENAME_7]:
+            with contextlib.suppress(FileNotFoundError):
+                os.remove(filename)
+
+
     # Constructor #
 
+    def testUseBcmAttribute(self):
+        self.assertFalse(self.canbus_raw.use_bcm)
+        self.assertTrue(self.canbus_bcm.use_bcm)
+
+    def testWriteToBcmAttribute(self):
+        self.assertRaises(AttributeError, setattr, self.canbus_raw, 'use_bcm', True)
+        self.assertRaises(AttributeError, setattr, self.canbus_raw, 'use_bcm', False)
+        self.assertRaises(AttributeError, setattr, self.canbus_bcm, 'use_bcm', True)
+        self.assertRaises(AttributeError, setattr, self.canbus_bcm, 'use_bcm', False)
+
+    def testWriteToConfigAttribute(self):
+        self.assertRaises(AttributeError, setattr, self.canbus_raw, 'config', configuration.Configuration())
+        self.assertRaises(AttributeError, setattr, self.canbus_bcm, 'config', configuration.Configuration())
+
     def testConstructor(self):
         self.canbus_raw.caninterface.close()
 
         config = configuration.Configuration()
         a = canbus.CanBus(config, VIRTUAL_CAN_BUS_NAME, timeout=1.0)
-        self.assertEqual(a.caninterface._interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertEqual(a.caninterface.interfacename, VIRTUAL_CAN_BUS_NAME)
         a.caninterface.close()
-        self.assertEqual(a.caninterface._interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertEqual(a.caninterface.interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertFalse(a.use_bcm)
 
         b = canbus.CanBus(config, VIRTUAL_CAN_BUS_NAME, timeout=1.0)
-        self.assertEqual(b.caninterface._interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertEqual(b.caninterface.interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertFalse(a.use_bcm)
         b.caninterface.close()
-        self.assertEqual(b.caninterface._interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertEqual(b.caninterface.interfacename, VIRTUAL_CAN_BUS_NAME)
 
         c = canbus.CanBus(config, VIRTUAL_CAN_BUS_NAME, timeout=1.0, use_bcm=True)
-        self.assertEqual(c.caninterface._interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertEqual(c.caninterface.interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertFalse(a.use_bcm)
         c.caninterface.close()
-        self.assertEqual(c.caninterface._interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertEqual(c.caninterface.interfacename, VIRTUAL_CAN_BUS_NAME)
 
     def testReadConfigFromFile(self):
         """See test_configuration.py and others for more complete configuration read testing"""
         bus = canbus.CanBus.from_kcd_file(self.input_filename, VIRTUAL_CAN_BUS_NAME)
+        self.assertFalse(bus.use_bcm)
 
-        self.assertEqual(bus.caninterface._interfacename, VIRTUAL_CAN_BUS_NAME)
-        self.assertEqual(bus._configuration.busname, "Mainbus")
+        self.assertEqual(bus.caninterface.interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertEqual(bus.config.busname, "Mainbus")
 
-        self.assertEqual(bus._configuration.framedefinitions[1].frame_id, 1)
-        self.assertEqual(bus._configuration.framedefinitions[1].frame_id, 1)
-        self.assertEqual(bus._configuration.framedefinitions[1].name, 'testframedef1')
+        self.assertEqual(bus.config.framedefinitions[1].frame_id, 1)
+        self.assertEqual(bus.config.framedefinitions[1].frame_id, 1)
+        self.assertEqual(bus.config.framedefinitions[1].name, 'testframedef1')
 
         bus2 = canbus.CanBus.from_kcd_file(self.input_filename, VIRTUAL_CAN_BUS_NAME, use_bcm=True)
-        self.assertEqual(bus2.caninterface._interfacename, VIRTUAL_CAN_BUS_NAME)
-        self.assertEqual(bus2._configuration.busname, "Mainbus")
+        self.assertEqual(bus2.caninterface.interfacename, VIRTUAL_CAN_BUS_NAME)
+        self.assertEqual(bus2.config.busname, "Mainbus")
 
     def testRepr(self):
         result = repr(self.canbus_raw)
@@ -122,13 +150,6 @@ class TestCanBus(unittest.TestCase):
         result = self.canbus_raw.get_descriptive_ascii_art()
         print('\n\n' + result)  # Check the output manually
 
-    # def testGetAllSignalNames(self):
-    #     # Input or output? Why?
-    #     return
-    #     result = self.canbus_raw.get_all_signalnames()
-    #     self.assertIn('testsignal1', result)
-    #     self.assertIn('testsignal2', result)
-    #     self.assertIn('testsignal3', result)
 
     # Send signals #
 
@@ -524,5 +545,5 @@ if __name__ == '__main__':
 
             # Run a single test #
     # suite = unittest.TestSuite()
-    # suite.addTest(TestCanBus("testReceiveNoData"))
+    # suite.addTest(TestCanBus("testReceiveAllSentFramesRaw"))
     # unittest.TextTestRunner(verbosity=2).run(suite)
